import sqlite3
import os
import sys
from typing import Tuple

class ImageHandler:

    def __init__(self, databasePath : str = "database.db", tableName : str = "images"):
        self.databasePath = databasePath
        self.tableName = tableName
        self.startConnection()

    def startConnection(self) -> None:
        """
        ### Explanation
        Starts the connection with SQLite Database.
        """

        self.connection = sqlite3.connect(f"{self.databasePath}")
        self.cursor = self.connection.cursor()
        self.cursor.execute(f"CREATE TABLE IF NOT EXISTS {self.tableName} (Name TEXT, Data BLOB, Type TEXT)")
        self.connection.commit()

    def imageSelector(self, path : str = None) -> Tuple[bytes, str]:
        """
        ### Explanation
        Selects an image and returns the image's bytes content and extension type.
        
        -------------------------------------------------
        ### Parameters
        - path[str]: The path of the picture you choose.

        -------------------------------------------------
        ### Returns
        - bytesContent[bytes]
        - extensionType[str]
        -------------------------------------------------
        """

        if path == None:
            raise ValueError("Path can't left blank!")

        with open(path, "rb") as file:
            bytesContent = file.read()

        extensionType = os.path.splitext(path)[1][1:]
        return bytesContent, extensionType

    def addImage(self, imageName : str, imageBytes : bytes, extensionType : str = "png") -> None:
        """
        ### Explanation
        Adds an image to database.

        -------------------------------------------------
        ### Parameters
        - imageName[str]:   Save name of image.
        - imageBytes[bytes]: Bytes of new image.
        - extensionType[str]: Extension type like png, jpg or something...
        -------------------------------------------------
        """

        if self.isImageExists(imageName):
            raise ValueError("There is an image with this name!")
        try:
            self.cursor.execute(f"INSERT INTO {self.tableName} VALUES (?, ?, ?)", (imageName, imageBytes, extensionType))
            self.connection.commit()
            print("Image successfully saved!")
        except Exception as error:
            print(error)

    def getSaveImage(self, imageName : str = None, savePath : str = "savedImage") -> None:
        """
        ### Explanation
        Saves the previously saved image in the database as an image to the given path.

        -------------------------------------------------
        ### Parameters
        - imageName[str]:   Name of image in the database.
        - savePath[str]:    Save path. (default: savedImage.png)
        -------------------------------------------------
        """
        if imageName == None:
            raise ValueError("Give me an image boss. (imageName parameter has not declared.)")

        path, extension = os.path.splitext(savePath)

        if extension != "":
            savePath = path

        try:
            self.cursor.execute(f"SELECT * FROM {self.tableName} WHERE Name = ?", (imageName,))
            data = self.cursor.fetchone()
            if data != None:
                with open(savePath + "." + data[2], "wb") as file:
                    file.write(data[1])
            else:
                print("There is no image with this name in the database!")
        except Exception as error:
            print(error)

        print(f"{imageName} saved successfully. Path:\n{savePath}")

    def isImageExists(self, imageName : str = None) -> bool:
        """
        ### Explanation
        Checks if image exists in database by image name.

        -------------------------------------------------
        ### Parameters
        - imageName[str]:   Name of image in the database.

        -------------------------------------------------
        ### Returns
        - isExists[bool]
        -------------------------------------------------
        """

        if imageName == None:
            raise ValueError("Image name can't left blank!")
        try:
            self.cursor.execute(f"SELECT * FROM {self.tableName} WHERE Name = ?", (imageName,))
            data = self.cursor.fetchone()
            if data != None:
                return True
            return False
        except Exception as error:
            print("Error occurred in isImageExists func. Error: " + str(error))

    def deleteImage(self, imageName : str = None) -> None:
        """
        ### Explanation
        Deletes image by name in database.

        -------------------------------------------------
        ### Parameters
        - imageName[str]:   Name of image in the database.
        -------------------------------------------------
        """

        if imageName == None:
            raise ValueError("Image name can't left blank!")
        
        if not self.isImageExists(imageName):
            raise NameError("There is no image with this name in the database!")
        
        try:
            self.cursor.execute(f"DELETE FROM {self.tableName} WHERE Name = ?", (imageName,))
            self.connection.commit()
            print("Image successfully deleted.")
        except Exception as error:
            print(error)

    def updateImage(self, imageName : str = None, newImageBytes : bytes = None, newExtensionType : str = None) -> None:
        """
        ### Explanation
        Updates image by name in database.

        -------------------------------------------------
        ### Parameters
        - imageName[str]:   Name of image in the database.
        - newImageBytes[bytes]: Bytes of new image.
        - extensionType[str]: Extension type like png, jpg or something...
        -------------------------------------------------
        """

        if imageName == None:
            raise ValueError("Image name can't left blank!")
        
        if not self.isImageExists(imageName):
            raise NameError("There is no image with this name in the database!")

        if newImageBytes == None:
            raise ValueError("New image bytes can't left blank!")

        try:
            if newExtensionType == None:
                self.cursor.execute(f"UPDATE {self.tableName} SET Data = ? WHERE Name = ?", (newImageBytes, imageName))
                self.connection.commit()
            else:
                self.cursor.execute(f"UPDATE {self.tableName} SET Data = ? , Type = ? WHERE Name = ?", (newImageBytes, newExtensionType, imageName))
                self.connection.commit()
            print("Image updated successfully")
        except Exception as error:
            print(error)


from SQLiteImageHandler import ImageHandler
import argparse
from __init__ import __version__

parser = argparse.ArgumentParser(description="SQLite Image Handler.")
parser.add_argument("-v", "--version", action="version", version=f'SQLiteImageHandler {__version__}')
parser.add_argument("--database-path", help="database path.", dest="dbpath", type=str)
parser.add_argument("--table-name", help="table name of the database", dest="tablename", type=str)

addGroup = parser.add_argument_group(title="Add an Image", description="Adding an image to the database.")
addGroup.add_argument("-a", "--add-image", help="Adds an image to database. (Requires -sn and -ip)", action="store_true")
addGroup.add_argument("-sn", "--save-name", help="Save name of image.", type=str)
addGroup.add_argument("-ip", "--image-path", help="Path of the selected image", type=str)

saveGroup = parser.add_argument_group(title="Save an Image", description="Saving image from database to computer.")
saveGroup.add_argument("-s", "--save-image", help="Saves the previously saved image in the database as an image to the given path. (Requires -sdn and -sp)", action="store_true")
saveGroup.add_argument("-sdn", "--saved-name", help="Saved name of image in the database.", type=str)
saveGroup.add_argument("-sp", "--save-path", help="Save path. (default: savedImage.png)", default="savedImage.png")

updateGroup = parser.add_argument_group(title="Update an Image", description="Updating the image in the database.")
updateGroup.add_argument("-u", "--update-image", help="Updates image by name in database. (Requires -in and -uip)", action="store_true")
updateGroup.add_argument("-in", "--image-name", help="Saved name of image in the database.", type=str)
updateGroup.add_argument("-uip", "--update-image-path", help="Path of the selected image", type=str)

deleteGroup = parser.add_argument_group(title="Delete an Image", description="Deleting an image from database")
deleteGroup.add_argument("-d", "--delete", help="Deletes image by name in database. (Requires -in)", action="store_true")

otherGroup = parser.add_argument_group(title="Other")
otherGroup.add_argument("-c", "--check", help="Checks if image exists in database by image name. (Requires -in)", action="store_true")
otherGroup.add_argument("-is", "--image-selector", help="Selects an image and returns the image's bytes length and extension type. (Requires -ip)", action="store_true")
args = parser.parse_args()

if args.dbpath == None or args.tablename == None:
    parser.print_help()
else:
    handler = ImageHandler(databasePath=args.dbpath, tableName=args.tablename)
    if args.add_image:
        if args.save_name != None and args.image_path != None:
            handler.addImage(args.save_name, *handler.imageSelector(path=args.image_path))
        else:
            print("You need to use -sn and -ip")

    if args.save_image:
        if args.saved_name != None and args.save_path != None:
            saved_image = args.saved_name
            handler.getSaveImage(imageName = saved_image, savePath = args.save_path)
        else:
            print("You need to use -sdn and -sp")

    if args.update_image:
        if args.image_name != None and args.update_image_path != None:
            name = args.image_name
            path = args.update_image_path
            handler.updateImage(name, *handler.imageSelector(path=path))
        else:
            print("You need you use -in and -uip")

    if args.delete:
        if args.image_name != None:
            image_name = args.image_name
            handler.deleteImage(imageName=image_name)
        else:
            print("You need to use -in")

    if args.check:
        if args.image_name != None:
            image_name = args.image_name
            print(handler.isImageExists(imageName=image_name))
        else:
            print("You need to use -in")

    if args.image_selector:
        if args.image_path != None:
            path = args.image_path
            bytesOfImage, extensionType = handler.imageSelector(path=path)
            print("Bytes Length: " + str(len(bytesOfImage)) + "\nExtension Type: " + extensionType)


from setuptools import setup, find_packages

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Customer Service',
    'Intended Audience :: Developers',
    'Operating System :: Microsoft :: Windows',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
]

setup(
    name='SQLiteImageHandler',
    version="0.1.0",
    description='Simple to use image handler for python sqlite3.',
    long_description='This package helps you to save images to sqlite database and get images from sqlite database.',
    url='https://github.com/mozancetin/SQLite-Image-Handler',
    author='Mustafa Ozan Çetin',
    author_email='mustafaozancetin@gmail.com',
    license='MIT',
    classifiers=classifiers,
    keywords=['sqlite', 'python', 'sqlite-image', 'image-handler'],
    packages=find_packages(),
    install_requires=['']
)
